import java.io.*;
import java.util.*;

/** Klasse zum Bestellen eines Warenkorbes.
  * Hier werden die Lagerbestnde verndert, bzw. die Warenkrbe
  * an den Lagerbestand angepasst.
  *
  * @author Bernhard Fiser 0109815
  * @version 1.0
  */
public class BestellServlet
{
	/** Diese Methode bestellt den Warenkorb, d.h. der Lagerbestand
	  * wird heruntergesetzt. Die Methode ist 'synchronized' um
	  * durch das Multithreading mgliche, auftrende race-conditions
	  * zu verhindern. */
	public synchronized static int bestellen(Warenkorb wk, ArtikelVerwaltung av)
	{
		HashMap		hmap;
		Iterator	it;
		Artikel		a,
				b;
		int		preis;

		/* Neue Hashmap erstellen. Diese Hashmap dient zum Vormerken
		 * der Artikel, die aus dem Warenkorb entfernt werden nachdem
		 * die Bestellung erfolgt ist. */
		hmap = new HashMap();
		/* Gesamtpreiszhler auf 0 setzen */
		preis = 0;
		/* Iteration ber alle Waren aus dem Warenkorb */
		it = wk.values().iterator();
		while (it.hasNext())
		{
			/* Artikel aus dem Warenkorb holen */
			a = (Artikel) it.next();
			/* Zugehrigen Artikel aus dem Lagerbestand holen */
			b = av.getArtikel(a.getID().intValue());
			/* Sind noch gengend Artikel auf Lager? */
			if (b.getMenge() >= a.getMenge())
			{
				/* Gesamtpreis berechnen */
				preis += b.getPreis() * a.getMenge();
				/* Menge im Lagerbestand reduzieren */
				b.setMenge(b.getMenge() - a.getMenge());
				/* Artikel zum lschen vormerken */
				hmap.put(a.getID(), a);
			}
		}
		/* Nach Vernderung des Lagerbestandes, diesen Speichern */
		av.save();
		/* Iteration ber alle zum Lschen vorgemerkten Artikel */
		it = hmap.values().iterator();
		while (it.hasNext())
		{
			/* Artikel aus Hashmap holen ... */
			a = (Artikel) it.next();
			/* ... und aus dem Warenkorb lschen */
			wk.remove(a.getID());
		}

		/* Gesamtpreis zurckgeben */
		return(preis);
	}

	/** Diese Methode passt alle Mengen der Artikel im Warenkorb an die
	  * im Lager verfgbare Menge an. */
	public synchronized static void anpassen(Warenkorb wk, ArtikelVerwaltung av)
	{
		Iterator	it;
		Artikel		a,
				b;

		/* Iteration ber alle Artikel des Warenkorbes */
		it = wk.values().iterator();
		while (it.hasNext())
		{
			/* Artikel aus Warenkorb lesen */
			a = (Artikel) it.next();
			/* Artikel aus Lagerbestand lesen */
			b = av.getArtikel(a.getID().intValue());
			/* Falls Bestellmenge grer als Lagerbestand ... */
			if (b.getMenge() < a.getMenge())
			{
				/* ... Bestellmenge redurzieren */
				a.setMenge(b.getMenge());
			}
		}
	}
}
